program pepsi;
{$R+,Q+,O-}
{$APPTYPE CONSOLE}

uses
  SysUtils;

const
  max_stars = 33;
  max_bottles = max_stars * 3 - 2;
  max_digits = 1000;
  base = 10000;

type
  huge_number = array [0..max_digits] of integer;

var
  number_of_stars: integer;
  need_of_bottles: integer;
  factorial: array [0..max_stars-1] of huge_number;
  power: array [0..max_bottles-1] of huge_number;
  once: array [0..max_bottles-1, 0..max_stars-1] of huge_number;
  sum: array [0..max_bottles] of huge_number;
  denominator, numerator, quotient, remainder, gcd: huge_number;
  intpart, denopart, numepart: string;
  i, n, m: integer;

procedure Sweep(var a: huge_number);
var
  i, carry: integer;
begin
  i := 1;
  while i <= a[0] do
  begin
    if a[i] >= base then
    begin
      carry := a[i] div base;
      a[i] := a[i] mod base;
      if i < a[0] then
        Inc(a[i+1], carry)
      else
      begin
        Inc(a[0]);
        a[i+1] := carry;
      end;
    end;
    if a[i] < 0 then
    begin
      Dec(a[i+1]);
      Inc(a[i], base);
    end;
    Inc(i);
  end;
  while (a[0] > 1) and (a[a[0]] = 0) do Dec(a[0]);
end;

function Huge_Int(i: integer): huge_number;
begin
  result[0] := 1;
  result[1] := i;
  Sweep(result);
end;

function Huge_Plus_Huge(a, b: huge_number): huge_number;
var
  i: integer;
begin
  result[0] := a[0];
  if b[0] > a[0] then result[0] := b[0];
  for i := a[0]+1 to result[0] do a[i] := 0;
  for i := b[0]+1 to result[0] do b[i] := 0;
  for i := 1 to result[0] do result[i] := a[i] + b[i];
  Sweep(result);
end;

function Huge_Minus_Huge(a, b: huge_number): huge_number;
var
  i: integer;
begin
  for i := b[0]+1 to a[0] do b[i] := 0;
  result[0] := a[0];
  for i := 1 to a[0] do
    result[i] := a[i] - b[i];
  Sweep(result);
end;

function Huge_Times_Int(a: huge_number; b: integer): huge_number;
var
  i: integer;
begin
  result[0] := a[0];
  for i := 1 to a[0] do
    result[i] := a[i] * b;
  Sweep(result);
end;

function Huge_Times_Huge(a, b: huge_number): huge_number;
var
  i, j: integer;
begin
  result[0] := a[0] + b[0] - 1;
  for i := 1 to result[0] do result[i] := 0;
  for i := 1 to a[0] do
    for j := 1 to b[0] do
      Inc(result[i+j-1], a[i] * b[j]);
  Sweep(result);
end;

function Huge_Greater(a, b: huge_number): boolean;
var
  i: integer;
begin
  if a[0] < b[0] then result := false
  else if a[0] > b[0] then result := true
  else
  begin
    result := false;
    for i := a[0] downto 1 do
      if a[i] < b[i] then exit
      else if a[i] > b[i] then
      begin
        result := true;
        exit;
      end;
  end;
end;

function Huge_Div_Huge(a, b: huge_number; var q, r: huge_number): boolean;
var
  i, j: integer;
  savec, c: huge_number;
begin
  if Huge_Greater(b, a) then
  begin
    q := Huge_Int(0);
    r := a;
  end
  else
  begin
    q[0] := a[0] - b[0] + 1;
    r[0] := b[0] - 1;
    for i := 1 to b[0]-1 do r[i] := a[q[0]+i];
    for i := q[0] downto 1 do
    begin
      for j := r[0] downto 1 do r[j+1] := r[j];
      Inc(r[0]);
      r[1] := a[i];
      q[i] := r[r[0]] div (b[b[0]]+1);
      while q[i] < base do
      begin
        c := Huge_Times_Int(b, q[i]);
        if Huge_Greater(c, r) then Break;
        savec := c;
        Inc(q[i]);
      end;
      r := Huge_Minus_Huge(r, savec);
      Dec(q[i]);
    end;
  end;
  Sweep(q);
  Sweep(r);
  result := (r[0] = 1) and (r[1] = 0);
end;

function Huge_GCD(a, b: huge_number): huge_number;
var
  c, q: huge_number;
begin
  while (b[0] <> 1) or (b[1] <> 0) do
  begin
    c := a;
    a := b;
    Huge_Div_Huge(c, a, q, b);
  end;
  result := a;
end;

function Huge_To_Str(a: huge_number): string;
var
  i, j: integer;
begin
  result := '';
  for i := 1 to a[0] do
    for j := 1 to 4 do
    begin
      result := Chr(a[i] mod 10 + Ord('0')) + result;
      a[i] := a[i] div 10;
    end;
  while (result <> '0') and (result[1] = '0') do Delete(result, 1, 1);
end;

begin
  { Input the number of soccer stars }
  repeat

  Write('Number of soccer stars: ');
  Readln(number_of_stars);
  need_of_bottles := number_of_stars * 3 - 2;

  if number_of_stars = 0 then Break;

  { Compute auxiliary constants }
  factorial[0] := Huge_Int(1);
  for i := 1 to number_of_stars - 1 do
    factorial[i] := Huge_Times_Int(factorial[i-1], i);
  power[0] := Huge_Int(1);
  for i := 1 to need_of_bottles - 1 do
    power[i] := Huge_Times_Int(power[i-1], number_of_stars);

  { Count permutations in which every star appears once at least }
  once[0, 0] := Huge_Int(0);
  for m := 1 to number_of_stars - 1 do
  begin
    once[m, m] := factorial[m];
    for n := m+1 to need_of_bottles - 1 do
      once[n, m] := Huge_Times_Int(Huge_Plus_Huge(once[n-1,m-1], once[n-1,m]), m);
  end;

  { Count sum of the number of oal-permutations }
  sum[number_of_stars - 1] := Huge_Int(0);
  for n := number_of_stars to need_of_bottles do
    sum[n] := Huge_Plus_Huge(sum[n-1], Huge_Times_Int(
      Huge_Times_Huge(once[n-1, number_of_stars - 1], power[need_of_bottles - n]), n));

  { Compute differences of sums }
  for i := 1 to 2 * (number_of_stars - 1) do
    for n := need_of_bottles downto number_of_stars + i do
      sum[n] := Huge_Minus_Huge(Huge_Times_Int(sum[n], number_of_stars),
        Huge_Times_Int(sum[n-1], (i + 1) div 2));

  { Compute average number of bottles = denominator / numerator }
  denominator := sum[need_of_bottles];
  numerator := Huge_Times_Huge(power[need_of_bottles - 1],
    Huge_Times_Huge(factorial[number_of_stars - 1], factorial[number_of_stars - 1]));

  { Output answer }
  Writeln('Average number of bottles to get a complete set:');
  if Huge_Div_Huge(denominator, numerator, quotient, remainder) then
  begin
    intpart := Huge_To_Str(quotient);
    Writeln(intpart);
  end
  else
  begin
    intpart := Huge_To_Str(quotient);
    denominator := remainder;
    gcd := Huge_GCD(denominator, numerator);
    Huge_Div_Huge(denominator, gcd, quotient, remainder);
    denopart := Huge_To_Str(quotient);
    Huge_Div_Huge(numerator, gcd, quotient, remainder);
    numepart := Huge_To_Str(quotient);
    Writeln('': Length(intpart)+1, denopart);
    Write(intpart, ' ');
    n := Length(denopart);
    if Length(numepart) > n then n := Length(numepart);
    for i := 1 to n do Write('-');
    Writeln;
    Writeln('': Length(intpart)+1, numepart);
  end;

{  Write('Press enter to return...');
  Readln;}
  Writeln;

  until False;
end.
