#include <string>
#include <fstream>
#include <iostream>
#include <memory>
using namespace std;

const int 
	CUp = 0, 
	CDown = 1, 
	CLeft = 2, 
	CRight = 3,
	CHome = 4,
	CEnd = 5;
const int 
	TBase = 0, 
	TExpFrame = 1, 
	TFraction = 2, 
	TMatrix = 3;

char* strins(char* src, char* insstr)
{
	int sl = strlen(insstr);
	for (int i = strlen(src); i >= 0; i--)
		src[i+sl] = src[i];
	return strncpy(src, insstr, sl);
}

class Base
{
public:
	static Base *CursorExp;
    int w, h, AlignLine;
	char **Content;

	int ClassType;
	Base *Owner;
	Base()
	{
		ClassType = TBase;
		w = 0; 
		h = 1; 
		AlignLine = 0;
		Content = NULL;
		Owner = NULL;
	}
	~Base()
	{
		DeleteContent();
	}

	virtual void MoveCursor(int Dir) // in the inner or to the outer
	{
	}
	virtual void PassCursor(int Dir) // from outer to inner
	{
	}
	virtual char** Format()
	{
		return Content; 
	}

	void NewContent()
	{
		Content = new char*[h];
		for (int iy = 0; iy < h; iy++)
		{
			Content[iy] = new char[w+2];
			memset(Content[iy], 32, w);
			Content[iy][w] = 0;
			Content[iy][w+1] = 0;
		}
	}
	void DeleteContent()
	{
		if (Content != NULL)
		{
			for (int iy = 0; iy < h; iy++)
				delete Content[iy];
			delete Content;
			Content = NULL;
		}
	}
	void PutExp(int x, int y, Base *E)
	{
		for (int iy = 0; iy < E->h; iy++)
			memcpy(Content[iy+y] + x, E->Content[iy],strlen(E->Content[iy]));
	}
};

class ExpFrame: public Base
{
public:
	int CurPos;
	ExpFrame(int len=500)
	{
		ClassType = TExpFrame;
		Express = new char[len];
		Express[0] = 0;
		CurPos = 0; 
		ind = 0;
	}
	~ExpFrame()
	{
		delete Express;
	}

	void InsElement(char *str);
	void PassCursor(int Dir)
	{
		CursorExp = this;
		if (Dir == CLeft) 
			CurPos = strlen(Express);
		else
			CurPos = 0;
	}
	void MoveCursor(int Dir)
	{
		switch (Dir)
		{
		case CUp:
		case CDown:
			if (Owner != NULL)
				Owner->MoveCursor(Dir);
			break;
		case CLeft:
			if (CurPos > 0)
			{
				CurPos--;
				if (Express[CurPos] < 32)
					NonStr[Express[CurPos] - 1]->PassCursor(Dir);
			}
			else
				if (Owner != NULL)
					Owner->MoveCursor(Dir);
			break;
		case CRight:
			if (Express[CurPos])
			{
				if (Express[CurPos] < 32)
					NonStr[Express[CurPos] - 1]->PassCursor(Dir);
				else
					CurPos++;
			}
			else
				if (Owner != NULL)
					Owner->MoveCursor(Dir);
			break;
		case CHome:
			CurPos = 0;
			break;
		case CEnd:
			CurPos = strlen(Express);
			break;
		}
	}
	char** Format()
	{
		DeleteContent();
		int len = strlen(Express);
		int hup = 0, hdown = 1;
		w = 0;
		for (int i = 0; i < len; i++)
			if (Express[i] < 32)
			{
				int idns = Express[i]-1;
				NonStr[idns]->Format();
				if (NonStr[idns]->AlignLine > hup)
					hup = NonStr[idns]->AlignLine;
				if (NonStr[idns]->h - NonStr[idns]->AlignLine > hdown)
					hdown = NonStr[idns]->h - NonStr[idns]->AlignLine;
				w+=NonStr[idns]->w;
			}
			else
			{
				if (Express[i] == '-')
					w += 3;
				else
					w++;
			}
		h = hup + hdown;
		AlignLine = hup;
		NewContent();

		int x = 0;
		for (int i = 0; i < len; i++)
		{
			if (Express[i] < 32)
			{
				int idns = Express[i]-1;
				PutExp(x, hup - NonStr[idns]->AlignLine, NonStr[idns]);
				x+=NonStr[idns]->w;
			}
			else
				if (Express[i] == '-')
				{
					Content[hup][x+1] = '-';
					x += 3;
				}
				else
					Content[hup][x++] = Express[i];
		}
		return Content; 
	}
protected:
	int ind;
	char* Express;
	(Base*) NonStr[30];
};

class Fraction: public Base
{
public:
	Fraction()
	{
		ClassType = TFraction;
		num = new ExpFrame();
		num->Owner = this;
		den = new ExpFrame();
		den->Owner = this;
	}
	~Fraction()
	{
		delete num;
		delete den;
	}

	void MoveCursor(int Dir)
	{
		switch (Dir)
		{
		case CUp:
			if (CurExp == den)
			{
				CurExp = num;
				num->PassCursor(Dir);
			}
			else
				if (Owner != NULL)
					Owner->MoveCursor(Dir);
			break;
		case CDown:
			if (CurExp == num)
			{
				CurExp = den;
				den->PassCursor(Dir);
			}
			else
				if (Owner != NULL)
					Owner->MoveCursor(Dir);
			break;
		case CLeft:
			CursorExp = Owner;
			break;
		case CRight:
			CursorExp = Owner;
			((ExpFrame*)Owner)->CurPos++;
		}
	}
	void PassCursor(int Dir)
	{
		CurExp = num;
		num->PassCursor(Dir);
	}
	char** Format()
	{
		DeleteContent();
		num->Format();
		den->Format();
		if (num->w > den->w)
			w = num->w;
		else
			w = den->w;
		w+=2;
		h = num->h + den->h + 1;
		AlignLine = num->h;
		NewContent();

		PutExp((w - num->w)/2, 0, num);
		memset(Content[AlignLine], '-', w);
		PutExp((w - den->w)/2, num->h+1, den);

		return Content;
	}
protected:
	ExpFrame *CurExp, *num, *den;
};

class Matrix: public Base
{
public:
	Matrix(int r, int c)
	{
		ClassType = TMatrix;
		ew = r; eh = c;
		for (int iy = 0; iy < eh; iy++)
			for (int ix = 0; ix < ew; ix++)
			{
				Cell[iy][ix] = new ExpFrame();
				Cell[iy][ix]->Owner = this;
			}
	}
	~Matrix()
	{
		for (int iy = 0; iy < eh; iy++)
			for (int ix = 0; ix < ew; ix++)
				delete Cell[iy][ix];
	}

	void PassCursor(int Dir)
	{
		if (Dir == CLeft)
		{
			cy = (eh-1)/2;
			cx = ew-1;
		}
		else
		{
			cy = (eh-1)/2;
			cx = 0;
		}
		Cell[cy][cx]->PassCursor(Dir);
	}
	void MoveCursor(int Dir)
	{
		switch (Dir)
		{
		case CUp:
			if (cy > 0)
			{
				cy--;
				Cell[cy][cx]->PassCursor(Dir);
			}
			else
				if (Owner != NULL)
					Owner->MoveCursor(Dir);
			break;
		case CDown:
			if (cy < eh-1)
			{
				cy++;
				Cell[cy][cx]->PassCursor(Dir);
			}
			else
				if (Owner != NULL)
					Owner->MoveCursor(Dir);
			break;
		case CLeft:
			if (cx > 0)
			{
				cx--;
				Cell[cy][cx]->PassCursor(Dir);
			}
			else
				CursorExp = Owner;
			break;
		case CRight:
			if (cx < ew-1)
			{
				cx++;
				Cell[cy][cx]->PassCursor(Dir);
			}
			else
			{
				CursorExp = Owner;
				((ExpFrame*)Owner)->CurPos++;
			}
		}
	}
	char** Format()
	{
		DeleteContent();
		int ix, iy;
		int hup[10], hdown[10], wmax[10];
		memset(hup, 0, sizeof(hup));
		memset(hdown, 0, sizeof(hdown));
		memset(wmax, 0, sizeof(wmax));

		for (iy = 0; iy < eh; iy++)
			for (ix = 0; ix < ew; ix++)
			{
				Cell[iy][ix]->Format();
				if (wmax[ix] < Cell[iy][ix]->w)
					wmax[ix] = Cell[iy][ix]->w;
				if (hup[iy] < Cell[iy][ix]->AlignLine)
					hup[iy] = Cell[iy][ix]->AlignLine;
				if (hdown[iy] < Cell[iy][ix]->h - Cell[iy][ix]->AlignLine)
					hdown[iy] = Cell[iy][ix]->h - Cell[iy][ix]->AlignLine;
			}

		w = 1;
		for (ix = 0; ix < ew; ix++)
			w += (wmax[ix] + 1);
		h = 0;
		for (iy = 0; iy < eh; iy++)
			h += (hup[iy] + hdown[iy] + 1);
		h--;
		AlignLine = 0;
		for (iy = 0; iy < eh / 2; iy++)
			AlignLine += (hup[iy] + hdown[iy] + 1);
		if (eh&1)
			AlignLine += hup[eh/2];
		else
			AlignLine --;
		NewContent();

		int x, y = 0;
		for (iy = 0; iy < eh; y += (hup[iy]+hdown[iy]+1), iy++)
		{
			x = 1;
			Content[y+hup[iy]][0] = '[';
			Content[y+hup[iy]][w-1] = ']';
			for (ix = 0; ix < ew; x += (wmax[ix]+1), ix++)
				PutExp(x + (wmax[ix]-Cell[iy][ix]->w)/2, y + hup[iy]-Cell[iy][ix]->AlignLine, Cell[iy][ix]);
		}

		return Content;
	}

	void AddRow()
	{
		AddRowBefore(cy);
	}
	void AddCol()
	{
		AddColBefore(cx);
	}
	void AddRowBefore(int r)
	{
		for (int iy = eh-1; iy >= r; iy--)
			for (int ix = 0; ix < ew; ix++)
				Cell[iy+1][ix] = Cell[iy][ix];
		for (int ix = 0; ix < ew; ix++)
		{
			Cell[r][ix] = new ExpFrame();
			Cell[r][ix]->Owner = this;
		}
		eh++;
		Cell[cy][cx]->PassCursor(CLeft);
	}
	void AddColBefore(int c)
	{
		for (int iy = 0; iy < eh; iy++)
			for (int ix = ew-1; ix >= c; ix--)
				Cell[iy][ix+1] = Cell[iy][ix];
		for (int iy = 0; iy < eh; iy++)
		{
			Cell[iy][c] = new ExpFrame();
			Cell[iy][c]->Owner = this;
		}
		ew++;
		Cell[cy][cx]->PassCursor(CLeft);
	}
protected:
	int ew, eh, cx, cy;
	(ExpFrame*) Cell[10][10];
};

void ExpFrame::InsElement(char *str)
{
	if (strlen(str) == 1)
	{
		strins(Express+CurPos, str);
		CurPos++;
	}
	else
	{
		strins(Express+CurPos, " ");
		Express[CurPos] = ind+1;
		if (strcmp(str, "Matrix") == 0)
			NonStr[ind] = new Matrix(1, 1);
		else
		if (strcmp(str, "Fraction") == 0)
			NonStr[ind] = new Fraction();
		NonStr[ind]->Owner = this;
		ind++;
		MoveCursor(CRight);
	}
}

Base* Base::CursorExp = new ExpFrame();

int main()
{
	ExpFrame *Global = (ExpFrame*)Base::CursorExp;

	ifstream fin("maths.in");
	while(cin.good())
	{
		char cmd[80];
		cin >> cmd;
		if (strlen(cmd )== 0) break;
		if (strcmp(cmd, "AddRow") == 0 || strcmp(cmd, "AddCol") == 0)
		{
			Base* Father = Base::CursorExp;
			for (; (Father != NULL) && (Father->ClassType != TMatrix); Father = Father->Owner);
			if (Father != NULL)
				if (strcmp(cmd, "AddRow")== 0)
					((Matrix*)Father)->AddRow();
				else
					((Matrix*)Father)->AddCol();
		}
		else
		if (strcmp(cmd, "Left") == 0)
			Base::CursorExp->MoveCursor(CLeft);
		else
		if (strcmp(cmd, "Right") == 0)
			Base::CursorExp->MoveCursor(CRight);
		else
		if (strcmp(cmd, "Up") == 0)
			Base::CursorExp->MoveCursor(CUp);
		else
		if (strcmp(cmd, "Down") == 0)
			Base::CursorExp->MoveCursor(CDown);
		else
		if (strcmp(cmd, "Home") == 0)
			Base::CursorExp->MoveCursor(CHome);
		else
		if (strcmp(cmd, "End") == 0)
			Base::CursorExp->MoveCursor(CEnd);
		else
			((ExpFrame*)Base::CursorExp)->InsElement(cmd);
	}

	ofstream fout("maths.out");
	Global->Format();
	for (int iy = 0; iy < Global->h; iy++)
	{
		for(int i = strlen(Global->Content[iy])-1; i >= 0 && Global->Content[iy][i] == 32; i--);
		Global->Content[iy][i+1] = 0;
		fout << Global->Content[iy] << endl;
	}
	return 0;
}
